﻿/* ============================================================
* Author:       liubingsheng
* Time:         2016/11/29 14:52:25
* FileName:     RedisSystem
* Purpose:      Redis
* =============================================================*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Utility
{
    /// <summary>
    /// List扩展
    /// </summary>
    public static class ListExtension
    {
        static Random random = new Random();

        /// <summary>
        /// 打乱
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="lstValues"></param>
        public static List<T> Random<T>(this List<T> selfList)
        {
            List<T> lstRandoms = new List<T>();

            while (selfList.Count != 0)
            {
                int index = random.Next(selfList.Count);
                T item = selfList[index];
                lstRandoms.Add(item);
                selfList.RemoveAt(index);
            }

            selfList.AddRange(lstRandoms);

            return selfList;
        }

        public static HashSet<T> ToHashSet<T>(this List<T> selfList)
        {
            HashSet<T> setResults = new HashSet<T>();
            foreach(var item in selfList)
            {
                setResults.Add(item);
            }
            return setResults;
        }

        /// <summary>
        /// 复制
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <returns></returns>
        public static List<T> Copy<T>(this List<T> selfList)
        {
            List<T> lstResults = new List<T>();
            lstResults.AddRange(selfList);
            return lstResults;
        }

        /// <summary>
        /// 随机1个元素
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <returns></returns>
        public static T RandomOne<T>(this List<T> selfList)
        {
            if (selfList.Count == 0)
            {
                return default(T);
            }

            int index = random.Next(selfList.Count);
            return selfList[index];
        }

        /// <summary>
        /// 随机多个元素组成的新列表
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <returns></returns>
        public static List<T> RandomRange<T>(this List<T> selfList)
        {
            if (selfList.Count == 0)
            {
                return default(List<T>);
            }

            List<T> lst = new List<T>();
            foreach (var item in selfList)
            {
                if (random.NextDouble() >= 0.5)
                {
                    lst.Add(item);
                }
            }
            return lst;
        }

        /// <summary>
        /// 删除多个元素
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="lstValues"></param>
        /// <param name="collection"></param>
        public static List<T> RemoveRange<T>(this List<T> selfList, IEnumerable<T> collection)
        {
            foreach (var item in collection)
            {
                selfList.Remove(item);
            }

            return selfList;
        }

        /// <summary>
        /// 添加多个元素，不能添加重复元素
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <param name="collection"></param>
        /// <returns></returns>
        public static List<T> AddRangeUnique<T>(this List<T> selfList, IEnumerable<T> collection)
        {
            foreach (var item in collection)
            {
                if(selfList.Contains(item))
                {
                    continue;
                }
                selfList.Add(item);
            }

            return selfList;
        }

        /// <summary>
        /// 是否包含所有这些条目
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <param name="collection"></param>
        /// <returns></returns>
        public static bool ContainsAll<T>(this List<T> selfList, IEnumerable<T> collection)
        {
            foreach (var item in collection)
            {
                if (!selfList.Contains(item))
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// 是否包含所有这些条目
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <param name="collection"></param>
        /// <returns></returns>
        public static bool Contains<T>(this List<T> selfList, Func<T, bool> func)
        {
            foreach (var item in selfList)
            {
                if (func(item))
                {
                    return true;
                }
            }
            return false;
        }


        /// <summary>
        /// 如果条目存在，移动到最后
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <param name="item"></param>
        /// <returns></returns>
        public static List<T> MoveToEnd<T>(this List<T> selfList, T item)
        {
            if (selfList.Contains(item))
            {
                selfList.Remove(item);
                selfList.Add(item);
            }

            return selfList;
        }

        /// <summary>
        /// 找到第一个满足条件的条目
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <param name="match"></param>
        /// <returns></returns>
        public static T FindFirst<T>(this List<T> selfList, Predicate<T> match)
        {
            return selfList.Find(match);
        }

        /// <summary>
        /// 去重加入
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <param name="value"></param>
        public static void AddUnique<T>(this List<T> selfList, T value)
        {
            if(selfList.Contains(value))
            {
                return;
            }
            selfList.Add(value);
        }

        /// <summary>
        /// 向上排序
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        public static List<T> SortUp<T>(this List<T> selfList)
        {
            selfList.OrderBy(item => item);
            return selfList;
        }

        /// <summary>
        /// 向下排序
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        public static List<T> SortDown<T>(this List<T> selfList)
        {
            selfList.OrderByDescending(item => item);
            return selfList;
        }

        /// <summary>
        /// 反转自己
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <returns></returns>
        public static List<T> ReverseSelf<T>(this List<T> selfList)
        {
            selfList.Reverse();
            return selfList;
        }

        /// <summary>
        /// 合并list
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfCollection"></param>
        /// <returns></returns>
        public static List<T> Combine<T>(this IEnumerable<List<T>> selfCollection)
        {
            List<T> lstResults = new List<T>();
            foreach (var item in selfCollection)
            {
                lstResults.AddRange(item);
            }
            return lstResults;
        }

        /// <summary>
        /// 合并list
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="lstCollections"></param>
        /// <returns></returns>
        public static List<T> AddRanges<T>(this List<T> selfList, IEnumerable<List<T>> lstCollections)
        {
            foreach (var item in lstCollections)
            {
                selfList.AddRange(item);
            }
            return selfList;
        }

        public static List<T> AddCollection<T>(this List<T> selfList, IEnumerable<T> Collection)
        {
            foreach (var item in Collection)
            {
                selfList.Add(item);
            }
            return selfList;
        }

        /// <summary>
        /// 两个列表是否相等
        /// </summary>
        /// <param name="lstValues1"></param>
        /// <param name="lstValues2"></param>
        /// <returns></returns>
        public static bool Equal<T>(this List<T> lstValues1, List<T> lstValues2)
        {
            if(lstValues1.Count != lstValues2.Count)
            {
                return false;
            }

            for(int i=0; i<lstValues1.Count; i++)
            {
                if(!lstValues1[i].Equals(lstValues2[i]))
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// 随机1个元素并将它从原列表删除
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <returns></returns>
        public static T RandomOneAndDel<T>(this List<T> selfList)
        {
            if (selfList.Count == 0)
            {
                return default(T);
            }

            int index = random.Next(selfList.Count);
            T result = selfList[index];
            selfList.RemoveAt(index);
            return result;
        }

        /// <summary>
        /// 随机n个元素并将它从原列表删除
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <returns></returns>
        public static List<T> RandomAndDel<T>(this List<T> selfList, int n)
        {
            List<T> result = new List<T>();
            if (selfList.Count < n)
            {
                return result;
            }
            for (int i = 0; i < n; i++)
            {
                result.Add(selfList.RandomOneAndDel());
            }
            return result;
        }

        public static string MembersString<T>(this List<T> selfList)
        {
            string strResult = "";
            for(int i=0; i<selfList.Count; i++)
            {
                var item = selfList[i];
                if(i != 0)
                {
                    strResult += ",";
                }
                
                strResult += item.ToString();
            }
            return strResult;
        }

        /// <summary>
        /// 等待所有
        /// </summary>
        /// <typeparam name="TResult"></typeparam>
        /// <typeparam name="T1"></typeparam>
        /// <param name="lstParams"></param>
        /// <param name="func"></param>
        /// <returns></returns>
        public static List<TResult> WhenAll<TResult, T1>(this IEnumerable<T1> lstParams, Func<T1, TResult> func)
        {
            var lstTasks = new List<TResult>();
            foreach (var item in lstParams)
            {
                var task = func(item);
                lstTasks.Add(task);
            }

            var arrResults = lstTasks;
            return arrResults.ToList();
        }

        /// <summary>
        /// 等待所有
        /// </summary>
        /// <typeparam name="T1"></typeparam>
        /// <param name="lstParams"></param>
        /// <param name="func"></param>
        /// <returns></returns>
        public static async Task WhenAll<T1>(this IEnumerable<T1> lstParams, Func<T1, Task> func)
        {
            var lstTasks = new List<Task>();
            foreach (var item in lstParams)
            {
                var task = func(item);
                lstTasks.Add(task);
            }

            await Task.WhenAll(lstTasks);
        }

        /// <summary>
        /// 删除元素多个
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <returns></returns>
        public static List<T> Remove<T>(this List<T> selfList, T value, int count)
        {
            for(int i=0; i<count; i++)
            {
                selfList.Remove(value);
            }
            return selfList;
        }

        /// <summary>
        /// 添加元素多个
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="selfList"></param>
        /// <returns></returns>
        public static List<T> Add<T>(this List<T> selfList, T value, int count)
        {
            for (int i = 0; i < count; i++)
            {
                selfList.Add(value);
            }
            return selfList;
        }

        /// <summary>
        /// 元素数量
        /// </summary>
        /// <param name="selfList"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public static int ItemCount(this List<int> selfList, int value)
        {
            int count = 0;
            foreach(var item in selfList)
            {
                if(item == value)
                {
                    count++;
                }
            }
            return count;
        }
    }
}
